﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.Objects;
using System.Data;
using System.Linq.Expressions;
using PrismTutorial.DataLayer.ValidationBlock;

namespace PrismTutorial.DataLayer {
    public class Repository:BaseRepository,IRepository {

        #region Base Implementation

        private bool _disposed;

        public Repository(ADVConnection context) {
            this._context = context;
            this._contextReused = true;
        }

        #endregion

        #region IRepository Members

        public int Add<T>(T entity) {
            ADVConnection context = GetObjectContext();
            if (EntityValidator<T>.Validate(entity)) {
                context.AddObject(typeof(T).Name, entity);
                int result = context.SaveChanges();
                ReleaseObjectContextIfNotReused();
                return result;
            }
            return -1;
        }

        public long Count<T>() {
            ADVConnection context = GetObjectContext();
            var query = new ObjectQuery<T>(
                typeof(T).Name, 
                context, 
                MergeOption.NoTracking);
            int count = query.Count();
            ReleaseObjectContextIfNotReused();
            return count;
        }

        public long Count<T>(Expression<Func<T, bool>> expression) {
            ADVConnection context = GetObjectContext();
            var query = new ObjectQuery<T>(
                typeof(T).Name, 
                context, 
                MergeOption.NoTracking)
                .Where(expression);
            int count = query.Count();
            ReleaseObjectContextIfNotReused();
            return count;
        }

        public int Delete<T>(T entity) {
            ADVConnection context = GetObjectContext();
            object originalItem;
            EntityKey key = context.CreateEntityKey(typeof(T).Name, entity);
            if(context.TryGetObjectByKey(key, out originalItem)){
                context.DeleteObject(originalItem);
            }
            int result =  context.SaveChanges();
            ReleaseObjectContextIfNotReused();
            return result;
        }

        public IList<T> GetAll<T>() {
            ADVConnection context = GetObjectContext();
            IList<T> list = context
                .CreateQuery<T>(
                "[" + typeof(T).Name + "]")
                .ToList();
            ReleaseObjectContextIfNotReused();
            return list;
        }

        public IList<T> GetAll<T>(Expression<Func<T, bool>> expression) {
            ADVConnection context = GetObjectContext();
            IList<T> list = context
                .CreateQuery<T>(
                "[" + typeof(T).Name + "]")
                .Where(expression)
                .ToList();
            ReleaseObjectContextIfNotReused();
            return list;
        }

        public T GetSingle<T>(Expression<Func<T, bool>> expression) {
            ADVConnection context = GetObjectContext();
            T result = context
                .CreateQuery<T>(
                "[" + typeof(T).Name + "]")
                .Where(expression)
                .FirstOrDefault();
            ReleaseObjectContextIfNotReused();
            return result;
        }

        public int Update<T>(T entity) {
            ADVConnection context = GetObjectContext();
            object originalItem;
            EntityKey key = context.CreateEntityKey(typeof(T).Name, entity);
            if(context.TryGetObjectByKey(key, out originalItem)){
                context.ApplyPropertyChanges(typeof(T).Name,entity);
            }
            int result = context.SaveChanges();
            ReleaseObjectContextIfNotReused();
            return result;
        }

        #endregion

        #region Disposable

        public void Dispose() {
            DisposeObject(true);
            GC.SuppressFinalize(this);
        }

        ~Repository() {
            DisposeObject(false);
        }

        private void DisposeObject(bool disposing) {
            if (_disposed) {
                return;
            }
            if (disposing) {
                if (_context != null) {
                    _context.Dispose();
                }
                _disposed = true;
            }
        }

        #endregion
    }
}
